/*
 * Copyright 1993-2010 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property and 
 * proprietary rights in and to this software and related documentation. 
 * Any use, reproduction, disclosure, or distribution of this software 
 * and related documentation without an express license agreement from
 * NVIDIA Corporation is strictly prohibited.
 *
 * Please refer to the applicable NVIDIA end user license agreement (EULA) 
 * associated with this source code for terms and conditions that govern 
 * your use of this NVIDIA software.
 * 
 */


#include "../common/book.h"
#include "../common/cpu_bitmap.h"

#define DIMx 1400
#define DIMy 800
#define Crmin -1.8
#define Crmax 0.8
#define Cimax 1.1
#define Cimin -1.1


float color_random_palette[] = {
    0.0f,    0.0f,    0.0f,
    0.609f,  0.115f,  0.436f,
    0.327f,  0.483f,  0.844f,
    0.822f,  0.569f,  0.201f,
    0.435f,  0.602f,  0.223f,
    0.310f,  0.747f,  0.185f,
    0.597f,  0.770f,  0.761f,
    0.559f,  0.436f,  0.730f,
    0.359f,  0.583f,  0.152f,
    0.483f,  0.596f,  0.789f,
    0.559f,  0.861f,  0.639f,
    0.195f,  0.548f,  0.859f,
    0.014f,  0.184f,  0.576f,
    0.771f,  0.328f,  0.970f,
    0.406f,  0.615f,  0.116f,
    0.676f,  0.977f,  0.133f,
    0.971f,  0.572f,  0.833f,
    0.140f,  0.616f,  0.489f,
    0.997f,  0.513f,  0.064f,
    0.945f,  0.719f,  0.592f,
    0.543f,  0.021f,  0.978f,
    0.279f,  0.317f,  0.505f,
    0.167f,  0.620f,  0.077f,
    0.347f,  0.857f,  0.137f,
    0.055f,  0.953f,  0.042f,
    0.714f,  0.505f,  0.345f,
    0.783f,  0.290f,  0.734f,
    0.722f,  0.645f,  0.174f,
    0.302f,  0.455f,  0.848f,
    0.225f,  0.587f,  0.040f,
    0.517f,  0.713f,  0.338f,
    0.053f,  0.959f,  0.120f,
    0.393f,  0.621f,  0.362f,
    0.673f,  0.211f,  0.457f,
    0.820f,  0.883f,  0.371f,
    1.0,     1.0,     1.0 
};


struct cuComplex {
    float   r;
    float   i;
    cuComplex( float a, float b ) : r(a), i(b)  {}
    float magnitude2( void ) { return r * r + i * i; }
    cuComplex operator*(const cuComplex& a) {
        return cuComplex(r*a.r - i*a.i, i*a.r + r*a.i);
    }
    cuComplex operator+(const cuComplex& a) {
        return cuComplex(r+a.r, i+a.i);
    }
};

int mandelbrot( int Cx, int Cy ) { 

    const float scale = 1.0;

    float Creal = scale * ( Crmin + (Crmax-Crmin)*Cx/(float)(DIMx-1) );
    float Cimg  = scale * ( Cimin + (Cimax-Cimin)*Cy/(float)(DIMy-1) );

    cuComplex C( Creal, Cimg ) ;
 
    cuComplex z( 0.0, 0.0 );

    int i = 0;
    for (i=0; i<256; i++) {
        z = z * z + C;
        if (z.magnitude2() > 4.0)
            return i;
    }

    return 255;
}

void kernel( unsigned char *ptr ){
    for (int y=0; y<DIMy; y++) {
        for (int x=0; x<DIMx; x++) {

            int offset = x + y * DIMx;

            int mandelbrotValue = mandelbrot( x, y );

            int entry = 3 * mandelbrotValue; 
            ptr[offset*4 + 0] = 255 * color_random_palette[entry];
            ptr[offset*4 + 1] = 255 * color_random_palette[entry + 1];
            ptr[offset*4 + 2] = 255 * color_random_palette[entry + 2];
            ptr[offset*4 + 3] = 255;
        }
    }
 }

int main( void ) {

    CPUBitmap bitmap( DIMx, DIMy );

    unsigned char *ptr = bitmap.get_ptr();

    kernel( ptr );

    bitmap.display_and_exit();
}

