/*
 * Copyright 1993-2010 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property and 
 * proprietary rights in and to this software and related documentation. 
 * Any use, reproduction, disclosure, or distribution of this software 
 * and related documentation without an express license agreement from
 * NVIDIA Corporation is strictly prohibited.
 *
 * Please refer to the applicable NVIDIA end user license agreement (EULA) 
 * associated with this source code for terms and conditions that govern 
 * your use of this NVIDIA software.
 * 
 */


#include "../common/book.h"
#include "../common/cpu_bitmap.h"

#define DIMx 1400
#define DIMy 800
#define Crmin -1.7
#define Crmax 0.8
#define Cimax 1.1
#define Cimin -1.1

struct cuComplex {
    float   r;
    float   i;
    cuComplex( float a, float b ) : r(a), i(b)  {}
    float magnitude2( void ) { return r * r + i * i; }
    cuComplex operator*(const cuComplex& a) {
        return cuComplex(r*a.r - i*a.i, i*a.r + r*a.i);
    }
    cuComplex operator+(const cuComplex& a) {
        return cuComplex(r+a.r, i+a.i);
    }
};

int mandelbrot( int Cx, int Cy ) { 

    const float scale = 1.0;

    float Creal = scale * ( Crmin + (Crmax-Crmin)*Cx/(float)(DIMx-1) );
    float Cimg  = scale * ( Cimin + (Cimax-Cimin)*Cy/(float)(DIMy-1) );

    cuComplex C( Creal, Cimg ) ;
 
    cuComplex z( 0.0, 0.0 );

    int i = 0;
    for (i=0; i<200; i++) {
        z = z * z + C;
        if (z.magnitude2() > 4.0)
            return 0;
    }

    return 1;
}

void kernel( unsigned char *ptr ){
    for (int y=0; y<DIMy; y++) {
        for (int x=0; x<DIMx; x++) {

            int offset = x + y * DIMx;

            int mandelbrotValue = mandelbrot( x, y );

            ptr[offset*4 + 0] = 255 * mandelbrotValue;
            ptr[offset*4 + 1] = 0;
            ptr[offset*4 + 2] = 0;
            ptr[offset*4 + 3] = 255;
        }
    }
 }

int main( void ) {

    CPUBitmap bitmap( DIMx, DIMy );

    unsigned char *ptr = bitmap.get_ptr();

    kernel( ptr );

    bitmap.display_and_exit();
}

